#!/usr/bin/env ruby

base_dir = File.dirname(__FILE__)

require "./#{base_dir}/sparql.rb"
require 'json'
require 'erb'
require 'fileutils'
require 'tempfile'
require 'open-uri'

ISQL = '/data/store/virtuoso7.2hv/bin/isql 20831 dba dba'
ISQL_OPT = 'VERBOSE=OFF BANNER=OFF PROMPT=OFF ECHO=OFF BLOBS=ON ERRORS=stderr'
VARIANT_DIR = '/data/store/rdf/variant'
OUTPUT_DIR = "#{VARIANT_DIR}/clinvar/current/additional_data"
class ClinVarReviewStatus

  def initialize
    FileUtils.mkdir_p("#{OUTPUT_DIR}")
    @base_dir = File.dirname(__FILE__)
  end

  def output_last_modified (clinvar_id, submission_list, output_ttl_file)
    has_last_modified_list = submission_list.find_all{|item| item[:last_modified] != ""}
    if has_last_modified_list.size == 0
      return nil
    else
      last_modified = has_last_modified_list.max_by {|item| item[:last_modified] }[:last_modified]
      output_ttl_file.puts "cvid:#{clinvar_id}\tdcterms:modified\t\"#{last_modified}\" ."
    end 
  end

  def output_clinical_significance (clinvar_id, mode, submission_list, output_ttl_file)
    if mode == "guideline"
      submission_list = submission_list.find_all{|sub_info| sub_info[:review_status] == "practice guideline"}
    elsif mode == "expert panel"
      submission_list = submission_list.find_all{|sub_info| sub_info[:review_status] == "reviewed by expert panel"}
    end  
    uniq_clinsig_list = submission_list.map {|sub|
      sub[:clinsig]
    }.uniq
    if mode == "conflicting interpretations"
      clinsig_text = ordered_clinsig_text(uniq_clinsig_list, true)
    else
      clinsig_text = ordered_clinsig_text(uniq_clinsig_list, false)
    end
    if clinsig_text.size > 0
      output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:clinicalSignificance\t\"#{clinsig_text}\" ."
    else
      puts "no clinsig #{clinvar_id}"
    end
  end


  def ordered_clinsig_text (clinsig_list, is_conflict)
     acmg_term = ["Pathogenic", "Likely pathogenic", "Benign", "Likely benign", "Uncertain significance"]
     if clinsig_list.size == 1
       return clinsig_list[0]
     else
       term_list = []
       if is_conflict
         #ACMGのtermはconflictionとしてまとめる
         term_list.push("Conflicting inmerpretations of pathogenicity")
         #ACMG以外のtermを追加する
         term_list += (clinsig_list - acmg_term).sort
       else
         #ACMGのtermを先に記述する
         acmg_term.each do |term|
           term_list.push(term) unless clinsig_list.find(term).nil? 
         end 
         #ACMG以外のtermを追加する
         term_list += (clinsig_list - acmg_term).sort
       end
       return term_list.join(", ")
     end
  end

  def output_clinvar_reviewstatus (output_ttl_file_path)
    query_file = "#{@base_dir}/sparql/clinvar_all_submission.rq"
    output_txt_file = "#{OUTPUT_DIR}/clinvar_all_submission.txt"
#    system(%Q[#{ISQL} #{ISQL_OPT} < #{query_file} > #{output_txt_file}])

    all_submission_list = []
    File.foreach(output_txt_file) do |line| 
      columns = line.split("^@")
      hash = {
        clinvar_id: columns[0].strip.chomp.split("/").last,
        submission_id: columns[2].strip.chomp,
        review_status: columns[3].strip.chomp,
        clinsig: columns[4].strip.chomp,
        submitter: columns[5].strip.chomp,
        last_modified: columns[6].strip.chomp
      }
      all_submission_list.push(hash)
    end

    output_ttl_file = File.open(output_ttl_file_path, "w")
    output_ttl_file.puts "@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> ."
    output_ttl_file.puts "@prefix dcterms: <http://purl.org/dc/terms/> ."
    output_ttl_file.puts "@prefix cvo: <http://purl.jp/bio/10/clinvar/> ."
    output_ttl_file.puts "@prefix cvid: <http://identifiers.org/clinvar/> ."
    output_ttl_file.puts
    all_submission_list.group_by {|row| row[:clinvar_id]}.each do |clinvar_id, submission_list|
      #has guideline
      #一つでもguidelineがあればguideline
      guideline_submission_list = submission_list.find_all{|sub_info| sub_info[:review_status] == "practice guideline"}
      if guideline_submission_list.size > 0
        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"practice guideline\" ."
        output_last_modified(clinvar_id, guideline_submission_list, output_ttl_file)
        output_clinical_significance(clinvar_id, "guideline", submission_list, output_ttl_file)
        next
      end

      #has expert panel
      #一つでもexpert panelがあればexpert panel
      expert_panel = submission_list.find_all{|sub_info| sub_info[:review_status] == "reviewed by expert panel"}
      if expert_panel.size > 1
        clisig = expert_panel.map {|row| row[:clinsig]}.uniq.join(",")
      end
      if expert_panel.size > 0
        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"reviewed by expert panel\" ."
        output_last_modified(clinvar_id, expert_panel, output_ttl_file)
        output_clinical_significance(clinvar_id, "expert panel", submission_list, output_ttl_file)
        next
      end

      #guideline, expert panelがなければ全てのsubmissionのうち最大のmodifiedを出力する
      output_last_modified(clinvar_id, submission_list, output_ttl_file)

      #no assertion_provided
      #全てがno assertion_providedだった場合はno assertion_provided
      no_assertion_provided = submission_list.find_all{|sub_info| sub_info[:review_status] == "no assertion provided"}
      if no_assertion_provided.size == submission_list.size
        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"no assertion provided\" ."
        output_clinical_significance(clinvar_id, "no assertion", submission_list, output_ttl_file)
        next
      end

      #no assertion_criteria provided
      #全てがno assertion_providedまたは"no assertion criteria provide"だった場合はno assertion_provided
      no_assertion_criteria_provided = submission_list.find_all{|sub_info| sub_info[:review_status] == "no assertion provided" || sub_info[:review_status] == "no assertion criteria provided"}
      if no_assertion_criteria_provided.size == submission_list.size
        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"no assertion criteria provided\" ."
        output_clinical_significance(clinvar_id, "no assertion criteria", submission_list, output_ttl_file)
        next
      end

      #conflicting on a submission
      #一つでもcriteria provided, conflicting interpretationsがあればcriteria provided, conflicting interpretations
      conflicting = submission_list.find_all{|sub_info| sub_info[:review_status] == "criteria provided, conflicting interpretations"}
      if conflicting.size > 0
        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"criteria provided, conflicting interpretations\" ."
        output_clinical_significance(clinvar_id, "conflicting interpretations", submission_list, output_ttl_file)
        next
      end

      submitter_list = submission_list.group_by {|sub_info| sub_info[:submitter]}
      #criteria providedを投稿しているsubmitterの数をカウント
      criteria_provided_submitter_num = 0
      submitter_list.each do |submitter, sub_list|
        conflicting = sub_list.find_all{|sub_info| sub_info[:review_status] == "criteria provided, conflicting interpretations"}
        criteria_provided = sub_list.find_all{|sub_info| sub_info[:review_status] == "criteria provided, single submitter"}
        if conflicting.size > 0 || criteria_provided.size > 0
          criteria_provided_submitter_num += 1 
        end
      end

#      #criteria providedを投稿しているsubmitterが1つであれば"criteria provided, single submitter"
#      if criteria_provided_submitter_num == 1
#        output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"criteria provided, single submitter\" ."
#        output_clinical_significance(clinvar_id, "single submitter", submission_list, output_ttl_file)
#      else #multiple submitters
        #submitter間でbenign/pathogenic/uncertain(ACMG term)と意見が割れていればcconflict
        acmg_term_count = {benign_count: 0, pathogenic_count: 0, uncertain_count: 0}
        submitter_list.each do |submitter, sub_list|
          benign = sub_list.find{|sub_info| 
            (sub_info[:clinsig] == "Benign" || sub_info[:clinsig] == "Likely benign") #&& sub_info[:review_status] == "criteria provided, single submitter"
          }
          pathogenic = sub_list.find{|sub_info|
            (sub_info[:clinsig] == "Pathogenic" || sub_info[:clinsig] == "Likely pathogenic") #&& sub_info[:review_status] == "criteria provided, single submitter"
          }
          uncertain = sub_list.find{|sub_info| 
            (sub_info[:clinsig] == "Uncertain significance") && sub_info[:review_status] #== "criteria provided, single submitter"
          }
          acmg_term_count[:benign_count] += 1 if !benign.nil?
          acmg_term_count[:pathogenic_count] += 1 if !pathogenic.nil?
          acmg_term_count[:uncertain_count] += 1 if !uncertain.nil?
        end
        #ACMG termでの主張が複数にまたがっていればconflict
        appearing_category = acmg_term_count.find_all{|k, count| count >= 1} 
        if appearing_category.size > 1 #conflict
          output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"criteria provided, conflicting interpretations\" ."
          output_clinical_significance(clinvar_id, "conflicting interpretations", submission_list, output_ttl_file)
        elsif criteria_provided_submitter_num == 1 #no conflict and 1submitter
          output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"criteria provided, single submitter\" ."
          output_clinical_significance(clinvar_id, "single submitter", submission_list, output_ttl_file)
        elsif criteria_provided_submitter_num > 1 #no conflict and 1submitter
          output_ttl_file.puts "cvid:#{clinvar_id}\tcvo:reviewStatus\t\"criteria provided, multiple submitters, no conflicts\" ."
          output_clinical_significance(clinvar_id, "no conflicts", submission_list, output_ttl_file)
        end
    #  end
    end 
    output_ttl_file.flush
    output_ttl_file.close
  end
end

clinvar = ClinVarReviewStatus.new
output_ttl_file = "#{OUTPUT_DIR}/clinvar_reviewstatus.ttl"
clinvar.output_clinvar_reviewstatus(output_ttl_file)
